#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "rssma.h"
#include "RSSMA_External_PM_Example.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rssma_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rssma_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
            rssma_close(io); \
            io = NULL;\
            return -1; \
        } \
    }
    
/* Globals */
static int panelHandle;

ViStatus  error;
ViSession io = NULL;
ViChar    error_message[256];
ViChar    error_buffer[256];

/* Local Functions */
int initialize (void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "RSSMA_External_PM_Example.uir", PANEL)) < 0)
        return -1;

    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
		{
		  	ViBoolean rfState;
			ViReal64  rfLevel, 
					  rfFrequency, 
					  pmExtDeviation;
			ViInt32   pmMode, 
					  pmState,
					  pmExtCoupling,
					  pmExtImpedance;
		
            if (io == NULL) 
            	if (initialize() < VI_SUCCESS) break;
            
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
            ProcessDrawEvents ();

            SetWaitCursor(1);

            GetCtrlVal (panelHandle, PANEL_RF_FREQ, &rfFrequency);
            GetCtrlVal (panelHandle, PANEL_RF_LEVEL, &rfLevel);
            GetCtrlVal (panelHandle, PANEL_RF_STATE, &rfState);
            GetCtrlVal (panelHandle, PANEL_PM_STATE, &pmState);
            GetCtrlVal (panelHandle, PANEL_PM_EXT_DEV, &pmExtDeviation);
            GetCtrlIndex (panelHandle, PANEL_PM_MODE, &pmMode);
            GetCtrlIndex (panelHandle, PANEL_PM_EXT_COUPLING, &pmExtCoupling);
            GetCtrlIndex (panelHandle, PANEL_PM_EXT_IMPEDANCE, &pmExtImpedance);
			
            CHECKERR (rssma_SetOutputState(io, rfState));
            
            if (rfState)
            {
               CHECKERR (rssma_confRFFreq(io, rfFrequency, 0.0, VI_FALSE, 0.0));
               CHECKERR (rssma_confRFLevel (io, rfLevel, 0.0, 30.0, VI_FALSE, 0.0,
                                            RSSMA_OUTPUT_AMODE_AUTO,
                                            RSSMA_POWER_LEVEL_RCL_EXCLUDE));
               
               CHECKERR (rssma_confPM (io, pmState, RSSMA_FM_SOURCE_EXTERNAL,
                                       pmMode, pmExtDeviation, 0.5, pmExtDeviation,
                                       pmExtCoupling,
                                       pmExtImpedance,
                                       RSSMA_FM_EXTERNAL_DIGITAL_FORMAT_BOFFSET));
            }
            
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);

            SetWaitCursor(0);
		} break;

    }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz SMA Signal Generator\n"
                              "instrument driver provided in the package.\n");
}

/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            if (io) rssma_close (io);
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;


    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rssma_init (instr_desc, id, rst, &io)) < 0 )
    {
        rssma_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);

    SetWaitCursor(0);
    return 0;
}

